# Energy Saving

IT4Innovations has implemented a set of energy saving measures on the supercomputing clusters. The measures are selected to minimize the performance impact and achieve significant cost, energy, and carbon footprint reduction effect.

The energy saving measures are effective as of **1.2.2023**.

## Karolina

### Measures

The CPU core and GPU streaming multiprocessors frequency limit is implemented for the Karolina supercomputer:

|Measure                                                  | Value   |
|---------------------------------------------------------|---------|
|Compute nodes **cn[001-720]**<br> CPU core frequency limit | 2.100 GHz |
|Accelerated compute nodes **acn[001-72]**<br> CPU core frequency limit | 2.600 GHz  |
|Accelerated compute nodes **acn[001-72]**<br> GPU SMs frequency limit | 1.290 GHz |

### Performance Impact

The performance impact depends on the [arithmetic intensity][1] of the executed workload.
The [arithmetic intensity][2] is a measure of floating-point operations (FLOPs) performed by a given code (or code section) relative to the amount of memory accesses (Bytes) that are required to support those operations. It is defined as a FLOP per Byte ratio (F/B).Arithmetic intensity is a characteristic of the computational algorithm.

In general, the processor frequency [capping][3] has low performance impact for memory bound computations (arithmetic intensity below the [ridge point][2]). For processor bound computations (arithmetic intensity above the [ridge point][2]), the impact is proportional to the frequency reduction.

On Karolina, runtime increase **up to 16%** is [observed][4] for arithmeticaly intensive CPU workloads and **up to 10%** for intensive GPU workloads. **No slowdown** is [observed][4] for memory bound workloads.

### Energy Efficiency

The energy efficiency in floating point operations per energy unit is increased by **up to 30%** for both the CPU and GPU workloads. The efficiency depends on the arithmetic intensity, however energy savings are always achieved.

## Barbora

None implemented yet.

## NVIDIA DGX-2

None implemented yet.

## Complementary Systems

None implemented yet.

[1]: https://en.wikipedia.org/wiki/Roofline_model
[2]: https://dl.acm.org/doi/10.1145/1498765.1498785
[3]: https://slovnik.seznam.cz/preklad/anglicky_cesky/capping
[4]: Energy_saving_Karolina.pdf
