# Molpro

## Introduction

MOLPRO is a complete software package used for accurate ab-initio quantum chemistry calculations.
For more information, see the [official webpage][a].

## License

MOLPRO software package is available only to users that have a valid license. Contact support to enable access to MOLPRO if you have a valid license appropriate for running on our cluster (e.g. academic research group licence, parallel execution).

To run MOLPRO, you need to have a valid license token present in `HOME/.molpro/token`. You can download the token from [MOLPRO website][b].

## Installed Versions

For the current list of installed versions, use:

```console
$ ml av Molpro
```

Compilation parameters are default:

| Parameter                          | Value        |
| ---------------------------------- | ------------ |
| max number of atoms                | 200          |
| max number of valence orbitals     | 300          |
| max number of basis functions      | 4095         |
| max number of states per symmetry  | 20           |
| max number of state symmetries     | 16           |
| max number of records              | 200          |
| max number of primitives           | maxbfn x [2] |

## Running

MOLPRO is compiled for parallel execution using MPI and OpenMP.
By default, MOLPRO reads the number of allocated nodes from Slurm
and launches a data server on one node.
On the remaining allocated nodes, compute processes are launched, one process per node.
You can modify this behavior by using the `-n`, `-t`, and `helper-server` options.
For more details, see the [MOLPRO documentation][c].

!!! note
    The OpenMP parallelization in MOLPRO is limited and has been observed to produce limited scaling. We therefore recommend to use MPI parallelization only. This can be achieved by passing the `--ntasks-per-node=128` and `--cpus-per-task=1` options to Slurm.

You are advised to use the `-d` option to point to a directory in SCRATCH file system.
MOLPRO can produce a large amount of temporary data during its run,
so it is important that these are placed in the fast scratch file system.

### Example jobscript

```bash
#!/bin/bash
#SBATCH --account=PROJECT_ID
#SBATCH --partition=qcpu
#SBATCH --nodes=1
#SBATCH --ntasks-per-node=128
#SBATCH --cpus-per-task=1

cd $SLURM_SUBMIT_DIR

# load Molpro module
ml Molpro/mpp-2024.1.0.linux_x86_64_mpipr

# create a directory in the SCRATCH filesystem
mkdir -p /scratch/project/PROJECT_ID/$SLURM_JOB_ID
cd /scratch/project/PROJECT_ID/$SLURM_JOB_ID

# copy an example input
cp $EBROOTMOLPRO/examples/caffeine_opt_diis.com .

# run Molpro with default options
molpro -d /scratch/project/PROJECT_ID/$SLURM_JOB_ID caffeine_opt_diis.com

exit 0
```

[a]: http://www.molpro.net/
[b]: https://www.molpro.net/licensee/?portal=licensee
[c]: http://www.molpro.net/info/2010.1/doc/manual/node9.html
