# ANSYS LS-DYNA

[ANSYSLS-DYNA][a] provides convenient and easy-to-use access to the technology-rich,
time-tested explicit solver without the need to contend
with the complex input requirements of this sophisticated program.
Introduced in 1996, ANSYS LS-DYNA capabilities have helped customers in numerous industries
to resolve highly intricate design issues.
ANSYS Mechanical users have been able to take advantage of complex explicit solutions
for a long time utilizing the traditional ANSYS Parametric Design Language (APDL) environment.
These explicit capabilities are available to ANSYS Workbench users as well.
The Workbench platform is a powerful, comprehensive, easy-to-use environment for engineering simulation.
CAD import from all sources, geometry cleanup, automatic meshing, solution,
parametric optimization, result visualization, and comprehensive report generation
are all available within a single fully interactive modern graphical user environment.

To run ANSYS LS-DYNA in batch mode, you can utilize/modify the default `ansysdyna.slurm` script
and execute it via the `sbatch` command:

```bash
#!/bin/bash
#SBATCH --nodes=5             # Request 5 nodes
#SBATCH --ntasks-per-node=128 # Request 128 MPI processes per node
#SBATCH --job-name=ANSYS-test # Job name
#SBATCH --partition=qcpu      # Partition name
#SBATCH --account=PROJECT_ID  # Account/project ID
#SBATCH --output=%x-%j.out    # Output log file with job name and job ID
#SBATCH --time=04:00:00       # Walltime


#!change the working directory (default is home directory)
#cd <working directory>
DIR=/scratch/project/PROJECT_ID/$SLURM_JOB_ID
mkdir -p "$DIR"
cd "$DIR" || exit

echo Running on host `hostname`
echo Time is `date`
echo Directory is `pwd`
echo This jobs runs on the following processors:
echo $SLURM_NODELIST

#! Counts the number of processors
NPROCS=$(scontrol show hostname $SLURM_NODELIST | wc -l)

echo This job has allocated $NPROCS nodes

ml ANSYS/2023R2-intel-2022.12

#### Set number of processors per host listing
procs_per_host=1
#### Create host list
hl=""
for host in $(scontrol show hostname $SLURM_NODELIST)
do
 if [ "$hl" = "" ]
 then hl="$host:$procs_per_host"
 else hl="${hl}:$host:$procs_per_host"
 fi
done

echo Machines: $hl

ansys211 -dis -lsdynampp i=input.k -machines $hl
```

[SVS FEM][b] recommends to utilize sources by keywords: nodes, ppn.
These keywords allows addressing directly the number of nodes (computers)
and cores (ppn) utilized in the job.
In addition, the rest of the code assumes such structure of allocated resources.

[a]: http://www.ansys.com/products/structures/ansys-ls-dyna
[b]: http://www.svsfem.cz
